@@ -10,9 +10,11 @@ our @ISA = qw(Rose::DB::Object::Metadata::Relationship);
 use Rose::Object::MakeMethods::Generic;
 use Rose::DB::Object::MakeMethods::Generic;
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
-__PACKAGE__->add_method_maker_argument_names
+__PACKAGE__->default_auto_method_types('get');
+
+__PACKAGE__->add_common_method_maker_argument_names
 (
   qw(class share_db key_columns)
 );
@@ -37,12 +39,18 @@ Rose::Object::MakeMethods::Generic->make_methods
   scalar => 
   [
     'foreign_key',
-    __PACKAGE__->method_maker_argument_names
+    __PACKAGE__->common_method_maker_argument_names
   ],
 );
 
-sub method_maker_class { 'Rose::DB::Object::MakeMethods::Generic' }
-sub method_maker_type  { 'object_by_key' }
+__PACKAGE__->method_maker_info
+(
+  get =>
+  {
+    class => 'Rose::DB::Object::MakeMethods::Generic',
+    type  => 'object_by_key',
+  },
+);
 
 sub type { 'one to one' }
 
@@ -66,16 +74,28 @@ sub _fk_or_self
   return $self->$method(@_);
 }
 
-sub make_method
+sub method_name
+{
+  my($self) = shift;
+  
+  if(my $fk = $self->foreign_key)
+  {
+    return $fk->method_name(@_);
+  }
+
+  return $self->SUPER::method_name(@_);
+}
+
+sub make_methods
 {
   my($self) = shift;
 
   if(my $fk = $self->foreign_key)
   {
-    return $fk->make_method(@_);
+    return $fk->make_methods(@_);
   }
 
-  return $self->SUPER::make_method(@_);
+  return $self->SUPER::make_methods(@_);
 }
 
 sub id
@@ -88,6 +108,18 @@ sub id
     join("\0", map { join("\1", lc $_, lc $column_map->{$_}) } sort keys %$column_map);
 }
 
+sub build_method_name_for_type
+{
+  my($self, $type) = @_;
+  
+  if($type eq 'get')
+  {
+    return $self->name;
+  }
+
+  return undef;
+}
+
 1;
 
 __END__
@@ -101,7 +133,7 @@ Rose::DB::Object::Metadata::Relationship::OneToOne - One to one table relationsh
   use Rose::DB::Object::Metadata::Relationship::OneToOne;
 
   $rel = Rose::DB::Object::Metadata::Relationship::OneToOne->new(...);
-  $rel->make_method(...);
+  $rel->make_methods(...);
   ...
 
 =head1 DESCRIPTION
@@ -110,10 +142,26 @@ Objects of this class store and manipulate metadata for relationships in which a
 
 This class inherits from L<Rose::DB::Object::Metadata::Relationship>. Inherited methods that are not overridden will not be documented a second time here.  See the L<Rose::DB::Object::Metadata::Relationship> documentation for more information.
 
+=head1 METHOD MAP
+
+=over 4
+
+=item C<get>
+
+L<Rose::DB::Object::MakeMethods::Generic>, L<object_by_key|Rose::DB::Object::MakeMethods::Generic/object_by_key>, ...
+
+=back
+
+See the L<Rose::DB::Object::Metadata::Relationship|Rose::DB::Object::Metadata::Relationship/"MAKING METHODS"> documentation for an explanation of this method map.
+
 =head1 OBJECT METHODS
 
 =over 4
 
+=item B<build_method_name_for_type TYPE>
+
+Return a method name for the relationship method type TYPE.  Returns the relationship's L<name|Rose::DB::Object::Metadata::Relationship/name> for the method type "get", undef otherwise.
+
 =item B<foreign_key [FK]>
 
 Get or set the L<Rose::DB::Object::Metadata::ForeignKey> object to which this object delegates all responsibility.
@@ -128,14 +176,6 @@ If passed a local column name LOCAL, return the corresponding column name in the
 
 Get or set a reference to a hash that maps local column names to foreign column names.
 
-=item B<method_maker_class>
-
-Returns L<Rose::DB::Object::MakeMethods::Generic>.
-
-=item B<method_maker_type>
-
-Returns C<object_by_key>.
-
 =item B<type>
 
 Returns "one to one".
